@@ -5,44 +5,45 @@ static void _mi_lambda(double *n, double *lambda, double *target, int *num,
     int *llx, int *lly, int *llz);
 
 /* shrinked mutual information, to be used for the asymptotic test. */
-SEXP shmi(SEXP x, SEXP y, SEXP lx, SEXP ly, SEXP length) {
+SEXP shmi(SEXP x, SEXP y, SEXP gsquare) {
 
 int i = 0, j = 0, k = 0;
 double **n = NULL, *ni = NULL, *nj = NULL;
-int *llx = INTEGER(lx), *lly = INTEGER(ly), *num = INTEGER(length);
+int llx = NLEVELS(x), lly = NLEVELS(y), num = LENGTH(x);
 int *xx = INTEGER(x), *yy = INTEGER(y);
-double lambda = 0, target = 1/(double)((*llx) * (*lly));
+double lambda = 0, target = 1/(double)(llx * lly);
 double *res = NULL;
 SEXP result;
 
   /* allocate and initialize result to zero. */
-  PROTECT(result = allocVector(REALSXP, 1));
+  PROTECT(result = allocVector(REALSXP, 2));
   res = REAL(result);
-  *res = 0;
+  res[0] = 0;
+  res[1] = (llx - 1) * (lly - 1);
 
   /* initialize the contingency table and the marginal frequencies. */
-  n = alloc2dreal(*llx, *lly);
-  ni = alloc1dreal(*llx);
-  nj = alloc1dreal(*lly);
+  n = alloc2dreal(llx, lly);
+  ni = alloc1dreal(llx);
+  nj = alloc1dreal(lly);
 
   /* compute the joint frequency of x and y. */
-  for (k = 0; k < *num; k++) {
+  for (k = 0; k < num; k++) {
 
     n[xx[k] - 1][yy[k] - 1]++;
 
   }/*FOR*/
 
   /* estimate the optimal lambda for the data. */
-  _mi_lambda((double *)n, &lambda, &target, num, llx, lly, NULL);
+  _mi_lambda((double *)n, &lambda, &target, &num, &llx, &lly, NULL);
 
   /* switch to the probability scale and shrink the estimates. */
-  for (i = 0; i < *llx; i++)
-    for (j = 0; j < *lly; j++)
-        n[i][j] = lambda * target + (1 - lambda) * n[i][j] / (*num);
+  for (i = 0; i < llx; i++)
+    for (j = 0; j < lly; j++)
+        n[i][j] = lambda * target + (1 - lambda) * n[i][j] / num;
 
   /* compute the marginals. */
-  for (i = 0; i < *llx; i++)
-    for (j = 0; j < *lly; j++) {
+  for (i = 0; i < llx; i++)
+    for (j = 0; j < lly; j++) {
 
     ni[i] += n[i][j];
     nj[j] += n[i][j];
@@ -50,14 +51,18 @@ SEXP result;
   }/*FOR*/
 
   /* compute the mutual information from the joint and marginal frequencies. */
-  for (i = 0; i < *llx; i++)
-    for (j = 0; j < *lly; j++) {
+  for (i = 0; i < llx; i++)
+    for (j = 0; j < lly; j++) {
 
       if (n[i][j] != 0)
-        *res += n[i][j] * log(n[i][j] / (ni[i] * nj[j]));
+        res[0] += n[i][j] * log(n[i][j] / (ni[i] * nj[j]));
 
     }/*FOR*/
 
+  /* rescale to match the G^2 test. */
+  if (isTRUE(gsquare))
+    res[0] *= 2 * num;
+
   UNPROTECT(1);
 
   return result;
@@ -66,48 +71,49 @@ SEXP result;
 
 /* shrinked conditional mutual information, to be used for the asymptotic
  * test. */
-SEXP shcmi(SEXP x, SEXP y, SEXP z, SEXP lx, SEXP ly, SEXP lz, SEXP length) {
+SEXP shcmi(SEXP x, SEXP y, SEXP z, SEXP gsquare) {
 
 int i = 0, j = 0, k = 0;
 double ***n = NULL, **ni = NULL, **nj = NULL, *nk = NULL;
-int *llx = INTEGER(lx), *lly = INTEGER(ly), *llz = INTEGER(lz);
-int *num = INTEGER(length);
+int llx = NLEVELS(x), lly = NLEVELS(y), llz = NLEVELS(z);
+int num = LENGTH(x);
 int *xx = INTEGER(x), *yy = INTEGER(y), *zz = INTEGER(z);
-double lambda = 0, target = 1/(double)((*llx) * (*lly) * (*llz));
+double lambda = 0, target = 1/(double)(llx * lly * llz);
 double *res = NULL;
 SEXP result;
 
   /* allocate and initialize result to zero. */
-  PROTECT(result = allocVector(REALSXP, 1));
+  PROTECT(result = allocVector(REALSXP, 2));
   res = REAL(result);
-  *res = 0;
+  res[0] = 0;
+  res[1] = (llx - 1) * (lly - 1) * llz;
 
   /* initialize the contingency table and the marginal frequencies. */
-  n = alloc3dreal(*llx, *lly, *llz);
-  ni = alloc2dreal(*llx, *llz);
-  nj = alloc2dreal(*lly, *llz);
-  nk = alloc1dreal(*llz);
+  n = alloc3dreal(llx, lly, llz);
+  ni = alloc2dreal(llx, llz);
+  nj = alloc2dreal(lly, llz);
+  nk = alloc1dreal(llz);
 
   /* compute the joint frequency of x, y, and z. */
-  for (k = 0; k < *num; k++) {
+  for (k = 0; k < num; k++) {
 
     n[xx[k] - 1][yy[k] - 1][zz[k] - 1]++;
 
   }/*FOR*/
 
   /* estimate the optimal lambda for the data. */
-  _mi_lambda((double *)n, &lambda, &target, num, llx, lly, llz);
+  _mi_lambda((double *)n, &lambda, &target, &num, &llx, &lly, &llz);
 
   /* switch to the probability scale and shrink the estimates. */
-  for (i = 0; i < *llx; i++)
-    for (j = 0; j < *lly; j++)
-      for (k = 0; k < *llz; k++)
-        n[i][j][k] = lambda * target + (1 - lambda) * n[i][j][k] / (*num);
+  for (i = 0; i < llx; i++)
+    for (j = 0; j < lly; j++)
+      for (k = 0; k < llz; k++)
+        n[i][j][k] = lambda * target + (1 - lambda) * n[i][j][k] / num;
 
   /* compute the marginals. */
-  for (i = 0; i < *llx; i++)
-    for (j = 0; j < *lly; j++)
-      for (k = 0; k < *llz; k++) {
+  for (i = 0; i < llx; i++)
+    for (j = 0; j < lly; j++)
+      for (k = 0; k < llz; k++) {
 
         ni[i][k] += n[i][j][k];
         nj[j][k] += n[i][j][k];
@@ -115,19 +121,19 @@ SEXP result;
 
       }/*FOR*/
 
-  for (k = 0; k < *llz; k++) {
+  for (k = 0; k < llz; k++) {
 
     /* check each level of the conditioning variable to avoid (again)
      * "divide by zero" errors. */
     if (nk[k] == 0)
       continue;
 
-    for (j = 0; j < *lly; j++) {
+    for (j = 0; j < lly; j++) {
 
-      for (i = 0; i < *llx; i++) {
+      for (i = 0; i < llx; i++) {
 
         if (n[i][j][k] > 0)
-          *res += n[i][j][k] * log( (n[i][j][k] * nk[k]) / (ni[i][k] * nj[j][k]) );
+          res[0] += n[i][j][k] * log( (n[i][j][k] * nk[k]) / (ni[i][k] * nj[j][k]) );
 
       }/*FOR*/
 
@@ -135,6 +141,10 @@ SEXP result;
 
   }/*FOR*/
 
+  /* rescale to match the G^2 test. */
+  if (isTRUE(gsquare))
+    res[0] *= 2 * num;
+
   UNPROTECT(1);
 
   return result;
