@@ -1,12 +1,18 @@
 #include "common.h"
 
 /* find out the partial ordering of the nodes of a DAG. */
-SEXP schedule(SEXP bn, SEXP root_nodes, SEXP debug) {
+SEXP schedule(SEXP bn, SEXP root_nodes, SEXP reverse, SEXP debug) {
 
 int *depth = NULL, *matched = NULL, *debuglevel = LOGICAL(debug);
 int d = 0, i = 0, j = 0, changed = 0, nnodes = 0;
+char *direction = NULL;
 SEXP nodes_data, nodes, try, children, ordering;
 
+  if (isTRUE(reverse))
+    direction = "parents";
+  else
+    direction = "children";
+
   /* get to the nodes' data in both 'bn' and 'bn.fit' objects. */
   nodes_data = getListElement(bn, "nodes");
 
@@ -20,9 +26,10 @@ SEXP nodes_data, nodes, try, children, ordering;
   /* allocate a status vector to trak the ordering of the nodes. */
   PROTECT(ordering = allocVector(INTSXP, nnodes));
   depth = INTEGER(ordering);
+  memset(depth, '\0', nnodes * sizeof(int));
 
   if (*debuglevel > 0)
-    Rprintf("* currently at depth 1 (root nodes).\n");
+    Rprintf("* currently at depth 1 (starting BFS).\n");
 
   /* set the root nodes as the starting point of the BFS. */
   PROTECT(try = match(nodes, root_nodes, 0));
@@ -54,7 +61,7 @@ SEXP nodes_data, nodes, try, children, ordering;
       if (depth[i] < d)
         continue;
 
-      children = getListElement(VECTOR_ELT(nodes_data, i), "children");
+      children = getListElement(VECTOR_ELT(nodes_data, i), direction);
 
       /* this node is a leaf, nothing to do, move along. */
       if (LENGTH(children) == 0)
@@ -70,7 +77,7 @@ SEXP nodes_data, nodes, try, children, ordering;
       for (j = 0; j < LENGTH(try); j++) {
 
         if (*debuglevel > 0)
-          Rprintf("  > got node %s, parent is %s.\n",
+          Rprintf("  > got node %s from %s.\n",
             NODE(matched[j] - 1), NODE(i));
 
         depth[matched[j] - 1] = d + 1;
@@ -97,3 +104,4 @@ SEXP nodes_data, nodes, try, children, ordering;
   return ordering;
 
 }/*SCHEDULE*/
+
